package org.apidesign.apifest08.currency;

import org.apidesign.apifest08.currency.ExchangeRateProvider.ExchangeRateRequest;
import org.apidesign.apifest08.currency.ExchangeRateProvider.ExchangeRateResponse;

/**
 * Exchange rate data source.
 * 
 * @author ked
 */
public final class ExchangeRateDataSource<AmountType, IdentifierType> {

  private final IdentifierType currencyAIdentifier;

  private final IdentifierType currencyBIdentifier;

  private final ExchangeRateProvider<AmountType, IdentifierType> exchangeRateProvider;

  private ExchangeRateDataSource(
      IdentifierType currencyAIdentifier,
      IdentifierType currencyBIdentifier,
      ExchangeRateProvider<AmountType, IdentifierType> exchangeRateProvider) {
        if (currencyAIdentifier == null ||
            currencyBIdentifier == null ||
            currencyAIdentifier.equals(currencyBIdentifier)) {
                throw new IllegalArgumentException("Inappropriate exchange rates' identifiers!");
        }
  this.currencyAIdentifier = currencyAIdentifier;
    this.currencyBIdentifier = currencyBIdentifier;
    this.exchangeRateProvider = exchangeRateProvider;
  }

  public IdentifierType getCurrencyAIdentifier() {
    return currencyAIdentifier;
  }

  public IdentifierType getCurrencyBIdentifier() {
    return currencyBIdentifier;
  }

  public ExchangeRateValue<AmountType, IdentifierType> getExchangeRate() {
    ExchangeRateRequest<AmountType, IdentifierType> request =
        new ExchangeRateRequest<AmountType, IdentifierType>();
    ExchangeRateResponse<AmountType, IdentifierType> response =
        new ExchangeRateResponse<AmountType, IdentifierType>();

    request.setCurrencyAIdentifier(currencyAIdentifier);
    request.setCurrencyBIdentifier(currencyBIdentifier);

    exchangeRateProvider.getExchangeRate(request, response);

    if (response.getExchangeRate().getCurrencyA().getIdentifier().equals(currencyAIdentifier) &&
        response.getExchangeRate().getCurrencyB().getIdentifier().equals(currencyBIdentifier)) {
      return response.getExchangeRate();
    } else {
      throw new IllegalStateException("Data source's provider returned inappropriate exchange rate!");
    }
  }

  public static <AmountType, IdentifierType> ExchangeRateDataSource<AmountType, IdentifierType> getExchangeRateDataSource(
      IdentifierType currencyAIdentifier,
      IdentifierType currencyBIdentifier,
      ExchangeRateProvider<AmountType, IdentifierType> exchangeRateProvider) {
    return new ExchangeRateDataSource<AmountType, IdentifierType>(
        currencyAIdentifier,
        currencyBIdentifier,
        exchangeRateProvider);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final ExchangeRateDataSource other =
        (ExchangeRateDataSource) obj;
    if (this.currencyAIdentifier != other.currencyAIdentifier &&
        (this.currencyAIdentifier == null || !this.currencyAIdentifier.equals(other.currencyAIdentifier))) {
      return false;
    }
    if (this.currencyBIdentifier != other.currencyBIdentifier &&
        (this.currencyBIdentifier == null || !this.currencyBIdentifier.equals(other.currencyBIdentifier))) {
      return false;
    }
    if (this.exchangeRateProvider != other.exchangeRateProvider &&
        (this.exchangeRateProvider == null || !this.exchangeRateProvider.equals(other.exchangeRateProvider))) {
      return false;
    }
    return true;
  }

  @Override
  public int hashCode() {
    int hash = 7;
    hash = 83 * hash + (this.currencyAIdentifier != null ? this.currencyAIdentifier.hashCode() : 0);
    hash = 83 * hash + (this.currencyBIdentifier != null ? this.currencyBIdentifier.hashCode() : 0);
    hash = 83 * hash + (this.exchangeRateProvider != null ? this.exchangeRateProvider.hashCode() : 0);
    return hash;
  }
}
